<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="klettres" version="6">
	<MenuBar>
		<Menu name="file" noMerge="1"><text>&amp;File</text>	
			<Action name ="play_new"/>
                        <Action name="play_again"/>
			<Action name="downloadnewstuff"/>
			<Action name="file_quit"/>
		</Menu>

		<Action name="levels"/>
		
		<Action name="languages"/>
		
		<Menu name="look_mode" noMerge="1"><text>L&amp;ook</text>
			<Action name="looks" />
			<Separator/>
			<Action name="mode_kid"/>
			<Action name="mode_grownup"/>
		</Menu>


		<Menu name="settings" noMerge="1"><text>&amp;Settings</text>
			<Action name="options_show_menubar" />
			<Merge name="StandardToolBarMenuHandler" />
			<Action name="options_show_statusbar"/>
			<Separator/>
			<Action name="options_configure_keybinding"/>
			<Action name="options_configure_toolbars"/>
			<Action name="options_configure"/>	
		</Menu>
	</MenuBar>
	<ToolBar name="mainToolbar" noMerge="1"><Text>Main</Text>
		<Action name="levels"/>
		<Separator/>
		<Action name="looks" />
		<Separator/>
		<Action name="mode_kid"/>
		<Action name="mode_grownup"/>
		<Separator/>
		<Action name="menubar"/>
		<Separator/>
                <Action name="play_again"/>
                <Separator />
		<Action name="file_quit"/>
	</ToolBar>
	<ToolBar name="specialCharToolbar" position="bottom" noMerge="1" iconText="icononly"><text>Special Characters</text>
	</ToolBar>
</gui>
